/*
 * Copyright (C) 2004 Pistooli <pistooli@lindowsmail.com>
 */

#include "kvisualboyadvance.h"

#include <qlabel.h>

#include <kmainwindow.h>
#include <klocale.h>

//KVisualBoyAdvance::KVisualBoyAdvance()
//    : KMainWindow( 0, "KVisualBoyAdvance" )
KVisualBoyAdvance::KVisualBoyAdvance()
{
	connect( pushButtonGetBinaryPath, SIGNAL( clicked() ), this, SLOT( getBinaryPath() ) );
	connect( pushButtonGetBiosFile, SIGNAL( clicked() ), this, SLOT( getBiosPath() ) );
	connect( pushButtonGetRomPath, SIGNAL( clicked() ), this, SLOT( getRomPath() ) );
	connect( pushButtonGetScreenShotDir, SIGNAL( clicked() ), this, SLOT( getScreenShotDir() ) );
	connect( pushButtonsaveDir, SIGNAL( clicked() ), this, SLOT( getSaveDir() ) );


	connect( pushButtonStart, SIGNAL( clicked() ), this, SLOT( startGame() ) );
	connect( pushButtonStop, SIGNAL( clicked() ), this, SLOT( stopGame() ) );
	connect( pushButtondefaultConfig, SIGNAL( clicked() ), this, SLOT( saveDefaultConfig() ) );
	connect( pushButtonExit, SIGNAL( clicked() ), this, SLOT( exitProgram() ) );
	connect( spinBoxFrameSkip, SIGNAL( valueChanged( int ) ), this, SLOT( setAutoFrameSkip() ) );
	connect( checkBoxFrameSkip, SIGNAL( stateChanged( int ) ), this, SLOT( setFrameSkipValue() ) );
	connect( spinBoxThrottle, SIGNAL( valueChanged( int ) ), this, SLOT( setThrottleState() ) );
	connect( checkBoxThrottle, SIGNAL( stateChanged( int ) ), this, SLOT( setThrottleValue() ) );

	initDefaults();
}

KVisualBoyAdvance::~KVisualBoyAdvance()
{
}

void KVisualBoyAdvance::initDefaults()
{
	proc = new QProcess( this );
	procConfigs = new QProcess( this );

	defaultConfig();

	//check and/or create config directory
	QDir configDir( QDir::homeDirPath() + "/.kvisualboyadvance/" );
	if ( !configDir.exists() ) // check for existing config directory
	{
		configDir.mkdir( QDir::homeDirPath() + "/.kvisualboyadvance/" );
	}
	readConfig();

	showConfig();

	showGameRoms();

	backupConfig(); // if we have a config file than lets back it up
}


void KVisualBoyAdvance::getBinaryPath()
{
//	qWarning( "getBinaryPath reached" ); //to be deleted later

QString tempString;
	tempString = binaryPath;

	if ( binaryPath == "" )
	binaryPath = getBinaryPathDir->getOpenFileName( QDir::homeDirPath() , "VisualBoyAdvance", this, "VisualBoyAdvance", "binary for playing ROMs");
	else
	binaryPath = getBinaryPathDir->getOpenFileName( QDir::homeDirPath() , "VisualBoyAdvance", this, "RVisualBoyAdvance", "binary for playing ROMs");

	if ( binaryPath.isNull() == true ) binaryPath = tempString;

	lineEditBinaryPath->setText( binaryPath ); // showing the path to VisualBoyAdvance
}

void KVisualBoyAdvance::getRomPath()
{
QString tempString;
	tempString = gamePath;

	if ( gamePath == "" )
	gamePath = getRomPathDir->getExistingDirectory( QDir::homeDirPath() , this, "ROMs directory", "select output directory", true, true );
	else
	gamePath = getRomPathDir->getExistingDirectory( gamePath , this, "ROMs directory", "select output directory", true, true );

	if ( gamePath.isNull() == true ) gamePath = tempString;
	else showGameRoms();

	lineEditRomPath->setText( gamePath ); // showing the directory for ROMs
}

void KVisualBoyAdvance::getScreenShotDir()
{
QString tempString;
	tempString = captureDir;

	if ( captureDir == "" )
	captureDir = dialogGetScreenShotDir->getExistingDirectory( QDir::homeDirPath() , this, "ScreenShots directory", "select screenshots directory", true, true );
	else
	captureDir = dialogGetScreenShotDir->getExistingDirectory( captureDir , this, "ScreenShots directory", "select screenshots directory", true, true );

	if ( captureDir.isNull() == true ) captureDir = tempString;

	lineEditScreenshotDir->setText( captureDir ); // showing the directory for ROMs
}

void KVisualBoyAdvance::getSaveDir()
{
QString tempString;
	tempString = saveDir;

	if ( saveDir == "" )
	saveDir = dialogGetSaveDir->getExistingDirectory( QDir::homeDirPath() , this, "Save state directory", "select save state directory", true, true );
	else
	saveDir = dialogGetSaveDir->getExistingDirectory( saveDir , this, "Save state directory", "select save state directory", true, true );

	if ( saveDir.isNull() == true ) saveDir = tempString;

	lineEditsaveDir->setText( saveDir ); // showing the directory for ROMs
}

void KVisualBoyAdvance::getBiosPath()
{
//	qWarning( "getBinaryPath reached" ); //to be deleted later

QString tempString;
	tempString = biosFile;

	if ( biosFile == "" )
	biosFile = getBiosFile->getOpenFileName( QDir::homeDirPath() , "*.*", this, "external BIOS", "external BIOS file ( zip files are not supported! )");
	else
	biosFile = getBiosFile->getOpenFileName( QDir::homeDirPath() , "*.*", this, "external BIOS", "external BIOS file ( zip files are not supported! )");

	if ( biosFile.isNull() == true ) biosFile = tempString;

	qWarning( biosFile );

	lineEditBiosFile->setText( biosFile ); // showing the path to VisualBoyAdvance
}


void KVisualBoyAdvance::showGameRoms()
{
	listViewRoms->clear();
	QDir romDir( gamePath );
	romDir.setNameFilter("*.gba *.GBA *.zip *.ZIP)");
	romDir.setFilter(QDir::Files | QDir::Readable);

	const QFileInfoList *list = romDir.entryInfoList();
        QFileInfoListIterator it( *list );
        QFileInfo *fi;

        while ( (fi = it.current()) != 0 ) {
		new QListViewItem( listViewRoms, fi->fileName().latin1() );
		++it;
        }
}

void KVisualBoyAdvance::startGame()
{
QString tempSelectedGame;
QString commandLineArgument;

if ( listViewRoms->childCount() != 0 ) // we do no execute if no items in the game list
{
	tempSelectedGame = listViewRoms->currentItem()->text( 0 );

	saveConfig(); // configuration was saved at the beginning...


	if ( !proc->isRunning() )
	{
		if ( tempSelectedGame != "0" ) // selection exist
		{
		qWarning( "to execute :" + binaryPath + " " + gamePath + tempSelectedGame ); //to be deleted later
		proc = new QProcess( this );
		proc->addArgument( binaryPath );
//		proc->addArgument( "--filter=" + comboBoxFilter->currentItem() );
//qWarning( "--filter=" + comboBoxFilter->currentItem() );
//		proc->addArgument( "--filter-tv-mode" );
//qWarning( "---filter=" + comboBoxFilter->currentItem() );
//		proc->addArgument( comboBoxGameSize->currentText() );
//		proc->addArgument( comboBoxShowSpeed->currentText() );
//		if ( configAutoFrameSkip == "0" )
//			{
//			proc->addArgument( "--no-auto-frameskip" );
//			proc->addArgument( "--frameskip=" + configFrameSkipValue );
//			}
//			else proc->addArgument( "--auto-frameskip" );
//		if ( configThrottle == "0" ) proc->addArgument( "--no-throttle" ); else
//			{
//			proc->addArgument( "--throttle=" + configThrottleValue );
//			}
//		if ( configRTC == "0" ) proc->addArgument( "--no-rtc" ); else proc->addArgument( "--rtc" );


//		proc->addArgument( "--yuv=" + comboBoxYUV->currentText() );

		proc->addArgument( "--config=" + configFile );

//		proc->addArgument( "--yuv=" + comboBoxYUV->currentText() );


		proc->addArgument( gamePath + "/" + tempSelectedGame );

		if ( !proc->start() )
		{
			QMessageBox::critical( 0,
				tr("Fatal error"),
				tr("Please set the path for VisualBoyAdvance binary."),
				tr("Quit") );
		}

		}
		else
		{
			QMessageBox::information( 0,
				tr("Error"),
				tr("Select a game to Play"),
				tr("OK") );
		}
	}
	else
	{
		QMessageBox::information( 0,
		tr("Error"),
		tr("Game is already running..."),
		tr("OK") );
	}

}

}

void KVisualBoyAdvance::exitProgram()
{
	if ( checkBoxSaveConfig->isChecked() )
	{
		//save config and exit
		qWarning( "exitProgram reached with save" );
		QFile file( configFileOld );
		if ( file.open( IO_ReadWrite ) )
		{
			QTextStream stream( &file );
  			stream << "# configuration file for KVisualBoyAdvance" << "\n";
  			stream << "# binaryPath" << "\n";
  			stream << binaryPath << "\n";
  			stream << "# gamePath" << "\n";
  			stream << gamePath << "\n";
  			stream << "# Video" "\n";
  			stream << comboBoxGameSize->currentItem() <<  "\n";
  			stream << "# Filters" << "\n";
  			stream << comboBoxFilter->currentItem() << "\n";
  			stream << "# Speed" << "\n";
  			stream << comboBoxShowSpeed->currentItem() << "\n";
  			stream << "# AutoFrameSkip" << "\n";
			stream << checkBoxFrameSkip->isChecked() << "\n";
  			stream << "# FrameSkip" << "\n";
			stream << spinBoxFrameSkip->value() << "\n";
  			stream << "# Throttle" << "\n";
			stream << checkBoxThrottle->isChecked() << "\n";
  			stream << "# ThrottleValue" << "\n";
			stream << spinBoxThrottle->value() << "\n";
  			stream << "# RTC" << "\n";
			stream << checkBoxRTC->isChecked() << "\n";
  			stream << "# YUV" << "\n";
  			stream << comboBoxYUV->currentItem() << "\n";
  			stream << "# end of configuration file" << "\n";
			file.close();
			saveConfig(); //saving new config as well
		}
		else
		{
		}
		this->close();
	}
	else
	{
		//just exit
		qWarning( "exitProgram reached without save" );
		restoreConfig(); // we need to restore the config file as we are exiting without save
		this->close();
	}
}

void KVisualBoyAdvance::stopGame()
{
	qWarning( "stopGame reached" );
	if ( proc->isRunning() )
	{
		proc->tryTerminate();
        	QTimer::singleShot( 500, proc, SLOT( kill() ) );
	}
}

void KVisualBoyAdvance::setAutoFrameSkip()
{
	if ( spinBoxFrameSkip->value() != 0 )
		if ( checkBoxFrameSkip->isChecked() ) checkBoxFrameSkip->setChecked( false );

}

void KVisualBoyAdvance::setFrameSkipValue()
{
	if ( checkBoxFrameSkip->isOn() )
		spinBoxFrameSkip->setValue( 0 );

}

void KVisualBoyAdvance::setThrottleState()
{
	if ( spinBoxThrottle->value() != 0 )
		if ( !checkBoxThrottle->isChecked() ) checkBoxThrottle->setChecked( true );
	if ( spinBoxThrottle->value() == 0 )
		if ( checkBoxThrottle->isChecked() ) checkBoxThrottle->setChecked( false );


}

void KVisualBoyAdvance::setThrottleValue()
{
	if ( !checkBoxThrottle->isOn() )
		spinBoxThrottle->setValue( 0 );

}


void KVisualBoyAdvance::saveConfig()
{
QString tempString;

//	if ( checkBoxSaveConfig->isChecked() )
//	{
		//save config and exit
		qWarning( "saveConfig reached" );
		QFile file( configFile );
		if ( file.open( IO_ReadWrite ) )
		{
			QTextStream stream( &file );
  			stream << "# All numeric values are in hexadecimal" << "\n";
  			stream << "# Use TAB or EQUAL sign to separate name from value" << "\n";
  			stream << "#" << "\n";
  			stream << "# Key configuration (all numbers are in hexadecimal!)" << "\n";
  			stream << "#" << "\n";
  			stream << "# Keys values are in the format YXXX where Y is the device number. 0 means" << "\n";
  			stream << "# keyborad and XXX is the SDL define for the desired key (read SDL_keysym.h)." << "\n";
  			stream << "#" << "\n";
  			stream << "# If Y is greater than 0, it means joystick number Y-1 and it uses the" << "\n";
  			stream << "# following format for XXX:" << "\n";
  			stream << "#" << "\n";
  			stream << "# - if XXX < 20, XXX is the axis number multiplied by 2. An even number means" << "\n";
  			stream << "#   movement to the negative side (on the X axis, it means left). An odd" << "\n";
  			stream << "#   number means movement to the positive side (on the X axis, it mean" << "\n";
  			stream << "#   right). For the Y axis, negative means up and positive means down." << "\n";
  			stream << "#   X axis is usally axis number 0 and Y is axis number 1." << "\n";
  			stream << "# - if 20 >= XXX > 30, then XXX is the HAT number multiplied by 4 plus the" << "\n";
  			stream << "#   direction: 0 for up, 1 for down, 2 for right and 3 for left. Example:" << "\n";
  			stream << "#   0021 is HAT 0 down, 0026 is HAT 1 right." << "\n";
  			stream << "# - if 80 >= XXX > 100, XXX is the joystick button number (XXX-080)." << "\n";
  			stream << "#" << "\n";
  			stream << "# Default key configuration is (value in parenthesis):" << "\n";
  			stream << "#" << "\n";
  			stream << "# Left          Left Arrow  (0114)" << "\n";
  			stream << "# Right         Right Arrow (0113)" << "\n";
  			stream << "# Up            Up Arrow    (0111)" << "\n";
  			stream << "# Down          Down Arrow  (0112)" << "\n";
  			stream << "# A             Z           (007a)" << "\n";
  			stream << "# B             X           (0078)" << "\n";
  			stream << "# L             A           (0061)" << "\n";
  			stream << "# R             S           (0073)" << "\n";
  			stream << "# Start         ENTER       (000d)" << "\n";
  			stream << "# Select        BACKSPACE   (0008)" << "\n";
  			stream << "# Speed up      SPACE       (0020)" << "\n";
  			stream << "# Capture       F12         (0125)" << "\n";
  			stream << "#" << "\n";
  			stream << "Joy0_Left=0114" << "\n";
  			stream << "Joy0_Right=0113" << "\n";
  			stream << "Joy0_Up=0111" << "\n";
  			stream << "Joy0_Down=0112" << "\n";
  			stream << "Joy0_A=007a" << "\n";
  			stream << "Joy0_B=0078" << "\n";
  			stream << "Joy0_L=0061" << "\n";
  			stream << "Joy0_R=0073" << "\n";
  			stream << "Joy0_Start=000d" << "\n";
  			stream << "Joy0_Select=0008" << "\n";
  			stream << "Joy0_Speed=0020" << "\n";
  			stream << "Joy0_Capture=0125" << "\n";
  			stream << "" << "\n";
  			stream << "# Motion support keys. Same format as above" << "\n";
  			stream << "#" << "\n";
  			stream << "# Default keys are (value in parenthesis):" << "\n";
  			stream << "#" << "\n";
  			stream << "# Left          Numeric Pad 4 (0104)" << "\n";
  			stream << "# Right         Numeric Pad 6 (0106)" << "\n";
  			stream << "# Up            Numeric Pad 8 (0108)" << "\n";
  			stream << "# Down          Numeric Pad 2 (0102)" << "\n";
  			stream << "#" << "\n";
  			stream << "Motion_Left=0104" << "\n";
  			stream << "Motion_Right=0106" << "\n";
  			stream << "Motion_Up=0108" << "\n";
  			stream << "Motion_Down=0102" << "\n";
  			stream << "" << "\n";
  stream << "# Frame skip setting. Allowed values are from 0 to 5 only." << "\n";
  			stream << "frameSkip="  + tempString.setNum( spinBoxFrameSkip->value() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Gameboy Frame skip setting. Allowed values are from 0 to 5 only." << "\n";
  stream << "gbFrameSkip=" + tempString.setNum( spinBoxFrameSkip->value() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Video setting. 0=1x, 1=2x, 2=3x, 3=4x." << "\n";
  stream << "video=" + tempString.setNum( comboBoxGameSize->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Use fullscreen mode. 0=false, any other value means true" << "\n";
  			stream << "fullScreen=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Disables MMX support" << "\n";
  if ( checkBoxdisableMMX->isOn() ) stream << "disableMMX=1" << "\n"; else stream << "disableMMX=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Use bios file. 0=false, any other value means true" << "\n";
  if ( checkBoxuseBios->isOn() ) stream << "useBios=1" << "\n"; else stream << "useBios=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Bios file full path and name (ZIP not supported)" << "\n";
  stream << "biosFile=" + biosFile << "\n";
  			stream << "" << "\n";
  			stream << "# Filter to use. 0=no filter, 1=TV Mode, 2=2xSaI, 3=Super 2xSaI, 4=Super Eagle" << "\n";
  			stream << "# 5=Pixelate, 6=Motion Blur, 7=AdvanceMAME Scale2x, 8=Simple 2x," << "\n";
  			stream << "# 9=Bilinear, A=Bilinear Plus, B=hq2x, C=lq2x" << "\n";
  stream << "filter=" + tempString.setNum( comboBoxFilter->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Disable status messages. 0=false, any other value means true" << "\n";
  			stream << "disableStatus=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Enable Gameboy border. 0=false, any other value means true" << "\n";
  			stream << "borderOn=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Gameboy emulator type. 0=automatic, 1=CGB/GBC, 2=SGB, 3=GB, 4=GBA, 5=SGB2" << "\n";
  			stream << "emulatorType=1" << "\n";
  			stream << "" << "\n";
  			stream << "# Enable washed colors. 0=false, any other value means true" << "\n";
  			stream << "colorOption=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Directories. Not setting one them makes the file go the rom directory." << "\n";
  			stream << "" << "\n";
  			stream << "# Save state directory" << "\n";
stream << "saveDir=" + saveDir << "\n";
  			stream << "" << "\n";
  			stream << "# Screen shot Capture directory" << "\n";
stream << "captureDir=" + captureDir << "\n";
  			stream << "" << "\n";
  			stream << "# Battery directory" << "\n";
  			stream << "#batteryDir=" << "\n";
  			stream << "" << "\n";
  			stream << "# Screen capture format" << "\n";
  			stream << "# 0=PNG, anything else for BMP" << "\n";
stream << "captureFormat=" + tempString.setNum( comboBoxcaptureFormat->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Sound quality" << "\n";
  			stream << "# 1=44 Khz, 2=22Khz, 4=11Khz" << "\n";
stream << "soundQuality=" + tempString.setNum( comboBoxsoundQuality->currentItem()+1 ) << "\n";
  			stream << "" << "\n";
  			stream << "# Sound Echo" << "\n";
  			stream << "# 0=false, anything else for true" << "\n";
if ( checkBoxsoundEcho->isOn() ) stream << "soundEcho=1" << "\n"; else stream << "soundEcho=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Sound Low pass filter" << "\n";
  			stream << "# 0=false, anything else for true" << "\n";
if ( checkBoxsoundLowPas->isOn() ) stream << "soundLowPass=1" << "\n"; else stream << "soundLowPass=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Sound reverse stereo" << "\n";
  			stream << "# 0=false, anything else for true" << "\n";
if ( checkBoxsoundReverse->isOn() ) stream << "soundReverse=1" << "\n"; else stream << "soundReverse=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Remove GBA intros" << "\n";
  			stream << "# 0=false, anything else for true" << "\n";
if ( checkBoxremoveIntros->isOn() ) stream << "removeIntros=1" << "\n"; else stream << "removeIntros=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Save Type" << "\n";
  			stream << "# 0=automatic, 1=EEPROM, 2=SRAM, 3=Flash, 4=EEPROM+Sensor, 5=NONE" << "\n";
stream << "saveType=" + tempString.setNum( comboBoxsaveType->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Flash size" << "\n";
  			stream << "# 0=64K Flash, 1=128K Flash" << "\n";
stream << "flashSize=" + tempString.setNum( comboBoxflashSize->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Sound volume" << "\n";
  			stream << "# 0=1x, 1=2x, 2=3x, 3=4x" << "\n";
stream << "soundVolume=" + tempString.setNum( comboBoxsoundVolume->currentItem() )<< "\n";
  			stream << "" << "\n";
  			stream << "# Interframe blending" << "\n";
  			stream << "# 0=none, 1=motion blur, 2=smart" << "\n";
  			stream << "ifbType=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Show emulation speed" << "\n";
  			stream << "# 0=none, 1=percentage, 2=detailed" << "\n";
stream << "showSpeed="  + tempString.setNum( comboBoxShowSpeed->currentItem() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Show speed in transparent mode" << "\n";
  			stream << "# 0=normal, anything else for transparent" << "\n";
  			stream << "showSpeedTransparent=1" << "\n";
  			stream << "" << "\n";
  			stream << "# Enable/Disable auto frameskip" << "\n";
  			stream << "# 0=disable, anything else to enable" << "\n";
if ( checkBoxFrameSkip->isChecked() ) stream << "autoFrameSkip=1" << "\n"; else stream << "autoFrameSkip=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Sets the desired throttle" << "\n";
  			stream << "# 0=disable, 5...1000 valid throttle speeds" << "\n";
stream << "throttle=" + tempString.setNum( spinBoxThrottle->value() ) << "\n";
  			stream << "" << "\n";
  			stream << "# Pauses the emulator when the window is inactive" << "\n";
  			stream << "# 0=disable, anything else to enable" << "\n";
  			stream << "pauseWhenInactive=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Enables AGBPrint support" << "\n";
  			stream << "# 0=disable, anything else to enable" << "\n";
  			stream << "agbPrint=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Enables GBA RTC support" << "\n";
  			stream << "# 0=disable, anything else to enable" << "\n";
  if ( checkBoxRTC->isChecked() ) stream << "rtcEnabled=1" << "\n"; else stream << "rtcEnabled=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Sound OFF flag" << "\n";
  			stream << "# 0=sound on, anything else turns off sound" << "\n";
  if ( checkBoxsoundOff->isChecked() ) stream << "soundOff=0" << "\n"; else stream << "soundOff=1" << "\n";
  			stream << "" << "\n";
  			stream << "# Sound Enable" << "\n";
  			stream << "# Controls which channels are enabled: (add values)" << "\n";
  			stream << "#   1 - Channel 1" << "\n";
  			stream << "#   2 - Channel 2" << "\n";
  			stream << "#   4 - Channel 3" << "\n";
  			stream << "#   8 - Channel 4" << "\n";
  			stream << "# 100 - DirectSound A" << "\n";
  			stream << "# 200 - DirectSound B" << "\n";
  			stream << "# 30f=all enabled, 0=mute all" << "\n";
  			stream << "soundEnable=30f" << "\n";
  			stream << "" << "\n";
  			stream << "# Controls automatic SGB border" << "\n";
  			stream << "# 0=disable, anything else enables automatic SGB border display" << "\n";
  			stream << "borderAutomatic=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Skip bios code" << "\n";
  			stream << "# 0=disable, anything else skips BIOS code" << "\n";
  			stream << "skipBios=0" << "\n";
  			stream << "" << "\n";
  			stream << "# The interval between the rewind saves" << "\n";
  			stream << "# Minimum of 0 seconds to disable rewind support," << "\n";
  			stream << "# Maximum of 60 minutes. Value in seconds (hexadecimal numbers)" << "\n";
  			stream << "rewindTimer=0" << "\n";
  			stream << "" << "\n";
  			stream << "# Enable enhanced save type detection" << "\n";
  			stream << "# 0=disable, anything else to enable (no longer used)" << "\n";
  			stream << "#enhancedDetection=1" << "\n";
			file.close();
		}
//	}

}

void KVisualBoyAdvance::defaultConfig()
{
	binaryPath = QDir::homeDirPath() + "/";
	gamePath = QDir::homeDirPath() + "/";
	configThrottle = "0";
	configYUV = "YV12";

	configFileOld = QDir::homeDirPath() + "/.kvisualboyadvance/kvisualboyadvance.conf";
	configFile = QDir::homeDirPath() + "/.kvisualboyadvance/VisualBoyAdvance.cfg";
	configFileBackup = QDir::homeDirPath() + "/.kvisualboyadvance/VisualBoyAdvance.backup";

// VisualBoyAdvance saved configurations
	configFrameSkipValue = "0"; //	frameSkip = "0";
	useBios = "0";
	biosFile= "none";
	disableMMX = "0";
	configVideo = "1";
	configFilters = "0";
	saveDir = QDir::homeDirPath() + "/"; // !!! save state of game
	captureDir = QDir::homeDirPath() + "/"; // !!! new type config
	captureFormat = "0"; // !!! new type 0 means PNG, anything else is BMP
	soundQuality = "2"; // sound quality 2 = 22 kHz
	soundEcho = "0";
	soundLowPass = "0";
	soundReverse = "0";
	removeIntros = "0";
	saveType = "0";
	flashSize = "0";
	soundVolume = "0";
	configSpeed = "0"; // showSpeed
	configAutoFrameSkip = "1"; // autoFrameSkip
	configThrottleValue ="0";
	configRTC = "0"; // rtcEnabled
	soundOff = "0"; // 0 value is actually sound ON

}

void KVisualBoyAdvance::saveDefaultConfig()
{
	defaultConfig(); //giving default values to variables

	showConfig(); // showing them in the GUI

	showGameRoms(); // showing game ROMS if any

	saveConfig(); // saving into config file

	backupConfig(); // backing up config file
}

void KVisualBoyAdvance::showConfig()
{

// !!! FINAL cut begins, READS FROM NEW CONFIG;
	if ( disableMMX == "0" ) checkBoxdisableMMX->setChecked( false ); else checkBoxdisableMMX->setChecked( true );
	if ( useBios == "0" ) checkBoxuseBios->setChecked( false ); else checkBoxuseBios->setChecked( true );
	lineEditBiosFile->setText( biosFile );
	spinBoxFrameSkip->setValue( configFrameSkipValue.toInt() );
	comboBoxGameSize->setCurrentItem( configVideo.toInt() );
	comboBoxFilter->setCurrentItem( configFilters.toInt() );
	lineEditsaveDir->setText( saveDir );
	lineEditScreenshotDir->setText( captureDir );
	comboBoxcaptureFormat->setCurrentItem( captureFormat.toInt() );
	comboBoxsoundQuality->setCurrentItem( soundQuality.toInt()-1 );
	if ( soundEcho == "0" ) checkBoxsoundEcho->setChecked( false ); else checkBoxsoundEcho->setChecked( true );
	if ( soundLowPass == "0" ) checkBoxsoundLowPas->setChecked( false ); else checkBoxsoundLowPas->setChecked( true );
	if ( soundReverse == "0" ) checkBoxsoundReverse->setChecked( false ); else checkBoxsoundReverse->setChecked( true );
	if ( removeIntros == "0" ) checkBoxremoveIntros->setChecked( false ); else checkBoxremoveIntros->setChecked( true );
	comboBoxsaveType->setCurrentItem( saveType.toInt() );
	comboBoxflashSize->setCurrentItem( flashSize.toInt() );
	comboBoxsoundVolume->setCurrentItem( soundVolume.toInt() );
	comboBoxShowSpeed->setCurrentItem( configSpeed.toInt() );
	if ( configAutoFrameSkip == "0" ) checkBoxFrameSkip->setChecked( false ); else checkBoxFrameSkip->setChecked( true );
	if ( configThrottleValue == "0" )
		{
		spinBoxThrottle->setValue( configThrottleValue.toInt() );
		checkBoxThrottle->setChecked( false );
		}
		else
		{
		spinBoxThrottle->setValue( configThrottleValue.toInt() );
		checkBoxThrottle->setChecked( true );
		}
	if ( configRTC == "0" ) checkBoxRTC->setChecked( false ); else checkBoxRTC->setChecked( true );
	if ( soundOff == "0" ) checkBoxsoundOff->setChecked( true ); else checkBoxsoundOff->setChecked( false );
// FINAL cut ends

	lineEditBinaryPath->setText( binaryPath );
	lineEditRomPath->setText( gamePath );
	comboBoxYUV->setCurrentItem( configYUV.toInt() );

}

void KVisualBoyAdvance::readConfig()
{

    QFile file( configFileOld );

    if ( file.open( IO_ReadOnly ) )
    {
	QTextStream stream( &file );
	QString line;
	while ( !stream.atEnd() ) {
		line = stream.readLine();
		if ( line == "# binaryPath" )
		{
			line = stream.readLine();
			binaryPath = line;
		}
		if ( line == "# gamePath" )
		{
			line = stream.readLine();
			gamePath = line;
		}
		if ( line == "# Throttle" )
		{
			line = stream.readLine();
			configThrottle = line;
		}
		if ( line == "# YUV" )
		{
			line = stream.readLine();
			configYUV = line;
		}
        }
        file.close();
    }

    QFile file1( configFile );

    if ( file1.open( IO_ReadOnly ) )
    {
	QTextStream stream( &file1 );
	QString line1;
	while ( !stream.atEnd() ) {
		line1 = stream.readLine();
		if ( line1 == "# Frame skip setting. Allowed values are from 0 to 5 only." )
		{
			line1 = stream.readLine();
			configFrameSkipValue = line1.right( 1 );
		}
		if ( line1.left(6) == "video=" )
		{
			configVideo = line1.right( 1 );
		}
		if ( line1 == "# Disables MMX support" )
		{
			line1 = stream.readLine();
			disableMMX = line1.right( 1 );
		}
		if ( line1 == "# Use bios file. 0=false, any other value means true" )
		{
			line1 = stream.readLine();
			useBios = line1.right( 1 );
		}
		if ( line1 == "# Bios file full path and name (ZIP not supported)" )
		{
			line1 = stream.readLine();
			biosFile = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(7) == "filter=" )
		{
			configFilters = line1.right( 1 );
		}
		if ( line1.left(8) == "saveDir=" )
		{
			saveDir = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(11) == "captureDir=" )
		{
			captureDir = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(14) == "captureFormat=" )
		{
			captureFormat = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(13) == "soundQuality=" )
		{
			soundQuality = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(10) == "soundEcho=" )
		{
			soundEcho = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(13) == "soundLowPass=" )
		{
			soundLowPass = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(13) == "soundReverse=" )
		{
			soundReverse = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(13) == "removeIntros=" )
		{
			removeIntros = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(13) == "removeIntros=" )
		{
			removeIntros = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(9) == "saveType=" )
		{
			saveType = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(10) == "flashSize=" )
		{
			flashSize = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(12) == "soundVolume=" )
		{
			soundVolume = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(10) == "showSpeed=" )
		{
			configSpeed = line1.section( "=", 1 , 1 );
		}
		// showSpeedTransparent
		if ( line1.left(14) == "autoFrameSkip=" )
		{
			configAutoFrameSkip = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(9) == "throttle=" )
		{
			configThrottleValue = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(11) == "rtcEnabled=" )
		{
			configRTC = line1.section( "=", 1 , 1 );
		}
		if ( line1.left(9) == "soundOff=" )
		{
			soundOff = line1.section( "=", 1 , 1 );
		}
	}
    file1.close();
    }
}

void KVisualBoyAdvance::backupConfig()
{

	QFile file( configFile );


	if ( file.exists() )
	{
	qWarning( "NOW: cp" + configFile + configFileBackup );
		procConfigs = new QProcess( this );
		procConfigs->addArgument( "cp" );
		procConfigs->addArgument( configFile );
		procConfigs->addArgument( configFileBackup );

		if ( !procConfigs->start() )
		{
			QMessageBox::critical( 0,
				tr("Fatal error"),
				tr("Could not backup Config File"),
				tr("Quit") );
		}
	}

}

void KVisualBoyAdvance::restoreConfig()
{

	QFile file( configFileBackup );

	if ( file.exists() )
	{
		procConfigs = new QProcess( this );
		procConfigs->addArgument( "cp" );
		procConfigs->addArgument( configFileBackup );
		procConfigs->addArgument( configFile );

		if ( !procConfigs->start() )
		{
			QMessageBox::critical( 0,
				tr("Fatal error"),
				tr("Could not restore Config File"),
				tr("Quit") );
		}
	}
}



#include "kvisualboyadvance.moc"
